import config from "../../config.js";
import createClient from "openapi-fetch";
import { ClientCredentials } from "simple-oauth2";
import { ApiClientError } from "./apiClientError.js";
const ATLAS_API_VERSION = "2025-03-12";
export class ApiClient {
    constructor(options) {
        this.getAccessToken = async () => {
            if (this.oauth2Client && (!this.accessToken || this.accessToken.expired())) {
                this.accessToken = await this.oauth2Client.getToken({});
            }
            return this.accessToken?.token.access_token;
        };
        this.authMiddleware = {
            onRequest: async ({ request, schemaPath }) => {
                if (schemaPath.startsWith("/api/private/unauth") || schemaPath.startsWith("/api/oauth")) {
                    return undefined;
                }
                try {
                    const accessToken = await this.getAccessToken();
                    request.headers.set("Authorization", `Bearer ${accessToken}`);
                    return request;
                }
                catch {
                    // ignore not availble tokens, API will return 401
                }
            },
        };
        this.errorMiddleware = {
            async onResponse({ response }) {
                if (!response.ok) {
                    throw await ApiClientError.fromResponse(response);
                }
            },
        };
        this.options = {
            ...options,
            baseUrl: options?.baseUrl || "https://cloud.mongodb.com/",
            userAgent: options?.userAgent ||
                `AtlasMCP/${config.version} (${process.platform}; ${process.arch}; ${process.env.HOSTNAME || "unknown"})`,
        };
        this.client = createClient({
            baseUrl: this.options.baseUrl,
            headers: {
                "User-Agent": this.options.userAgent,
                Accept: `application/vnd.atlas.${ATLAS_API_VERSION}+json`,
            },
        });
        if (this.options.credentials?.clientId && this.options.credentials?.clientSecret) {
            this.oauth2Client = new ClientCredentials({
                client: {
                    id: this.options.credentials.clientId,
                    secret: this.options.credentials.clientSecret,
                },
                auth: {
                    tokenHost: this.options.baseUrl,
                    tokenPath: "/api/oauth/token",
                },
            });
            this.client.use(this.authMiddleware);
        }
        this.client.use(this.errorMiddleware);
    }
    async getIpInfo() {
        const accessToken = await this.getAccessToken();
        const endpoint = "api/private/ipinfo";
        const url = new URL(endpoint, this.options.baseUrl);
        const response = await fetch(url, {
            method: "GET",
            headers: {
                Accept: "application/json",
                Authorization: `Bearer ${accessToken}`,
                "User-Agent": this.options.userAgent,
            },
        });
        if (!response.ok) {
            throw await ApiClientError.fromResponse(response);
        }
        return (await response.json());
    }
    // DO NOT EDIT. This is auto-generated code.
    async listClustersForAllProjects(options) {
        const { data } = await this.client.GET("/api/atlas/v2/clusters", options);
        return data;
    }
    async listProjects(options) {
        const { data } = await this.client.GET("/api/atlas/v2/groups", options);
        return data;
    }
    async createProject(options) {
        const { data } = await this.client.POST("/api/atlas/v2/groups", options);
        return data;
    }
    async getProject(options) {
        const { data } = await this.client.GET("/api/atlas/v2/groups/{groupId}", options);
        return data;
    }
    async listProjectIpAccessLists(options) {
        const { data } = await this.client.GET("/api/atlas/v2/groups/{groupId}/accessList", options);
        return data;
    }
    async createProjectIpAccessList(options) {
        const { data } = await this.client.POST("/api/atlas/v2/groups/{groupId}/accessList", options);
        return data;
    }
    async listClusters(options) {
        const { data } = await this.client.GET("/api/atlas/v2/groups/{groupId}/clusters", options);
        return data;
    }
    async createCluster(options) {
        const { data } = await this.client.POST("/api/atlas/v2/groups/{groupId}/clusters", options);
        return data;
    }
    async getCluster(options) {
        const { data } = await this.client.GET("/api/atlas/v2/groups/{groupId}/clusters/{clusterName}", options);
        return data;
    }
    async listDatabaseUsers(options) {
        const { data } = await this.client.GET("/api/atlas/v2/groups/{groupId}/databaseUsers", options);
        return data;
    }
    async createDatabaseUser(options) {
        const { data } = await this.client.POST("/api/atlas/v2/groups/{groupId}/databaseUsers", options);
        return data;
    }
}
//# sourceMappingURL=apiClient.js.map