import { z } from "zod";
import { AtlasToolBase } from "./atlasTool.js";
export class ListClustersTool extends AtlasToolBase {
    constructor() {
        super(...arguments);
        this.name = "atlas-list-clusters";
        this.description = "List MongoDB Atlas clusters";
        this.argsShape = {
            projectId: z.string().describe("Atlas project ID to filter clusters").optional(),
        };
    }
    async execute({ projectId }) {
        this.session.ensureAuthenticated();
        if (!projectId) {
            const data = await this.session.apiClient.listClustersForAllProjects();
            return this.formatAllClustersTable(data);
        }
        else {
            const project = await this.session.apiClient.getProject({
                params: {
                    path: {
                        groupId: projectId,
                    },
                },
            });
            if (!project?.id) {
                throw new Error(`Project with ID "${projectId}" not found.`);
            }
            const data = await this.session.apiClient.listClusters({
                params: {
                    path: {
                        groupId: project.id || "",
                    },
                },
            });
            return this.formatClustersTable(project, data);
        }
    }
    formatAllClustersTable(clusters) {
        if (!clusters?.results?.length) {
            throw new Error("No clusters found.");
        }
        const rows = clusters.results
            .map((result) => {
            return (result.clusters || []).map((cluster) => {
                return { ...result, ...cluster, clusters: undefined };
            });
        })
            .flat()
            .map((cluster) => {
            return `${cluster.groupName} (${cluster.groupId}) | ${cluster.name}`;
        })
            .join("\n");
        return {
            content: [
                { type: "text", text: `Here are your MongoDB Atlas clusters:` },
                {
                    type: "text",
                    text: `Project | Cluster Name
----------------|----------------
${rows}`,
                },
            ],
        };
    }
    formatClustersTable(project, clusters) {
        if (!clusters?.results?.length) {
            throw new Error("No clusters found.");
        }
        const rows = clusters.results
            .map((cluster) => {
            const connectionString = cluster.connectionStrings?.standard || "N/A";
            const mongoDBVersion = cluster.mongoDBVersion || "N/A";
            return `${cluster.name} | ${cluster.stateName} | ${mongoDBVersion} | ${connectionString}`;
        })
            .join("\n");
        return {
            content: [
                {
                    type: "text",
                    text: `Here are your MongoDB Atlas clusters in project "${project.name}" (${project.id}):`,
                },
                {
                    type: "text",
                    text: `Cluster Name | State | MongoDB Version | Connection String
----------------|----------------|----------------|----------------|----------------
${rows}`,
                },
            ],
        };
    }
}
//# sourceMappingURL=listClusters.js.map